
/**********************************************************************\
*                                                                      *
* Module      : tb_x261v_ccmw1.v              Last updated: 07/14/00   *
*                                                                      *
* Description : Testbench for common-clock FIFO controller in a        *
*               Virtex2 device, using source fifoctlr_ccmw1_v2,v.      *
*               which implements a 2044x9 Read, 511x36 Write FIFO.     *
*                                                                      *
*  Designer    : Nick Camilleri                                        *
*                                                                      *
*  Company     : Xilinx, Inc.                                          *
*                                                                      *
*  Disclaimer  : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY   *
*                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY      *
*                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR    *
*                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.        *
*                THEY ARE ONLY INTENDED TO BE USED BY XILINX           *
*                CUSTOMERS, AND WITHIN XILINX DEVICES.                 *
*                                                                      *
*                Copyright (c) 2000 Xilinx, Inc.                       *
*                All rights reserved                                   *
*                                                                      *
\**********************************************************************/

`timescale 1ns / 10ps

module test;

reg GSR;
`define GSR_SIGNAL test.GSR

reg clockin;
reg read_enable, write_enable;
wire [8:0] read_data;
reg [35:0] write_data;
reg temp;

wire full, empty;
wire [3:0] fifocount;

fifoctlr_ccmw1_v2 fifo_controller (.clock_in(clockin), 
                       .read_enable_in(read_enable), .read_data_out(read_data), 
                       .write_data_in(write_data), .write_enable_in(write_enable),
                       .full_out(full), .empty_out(empty), 
                       .fifocount_out(fifocount), .fifo_gsr_in(GSR) );

initial begin
   read_enable = 0;
   write_enable = 0;
   write_data = 'h0;
   test.fifo_controller.read_addr = 'h0;
   test.fifo_controller.read_addr_minor = 'h0;
   test.fifo_controller.write_addr = 'h0;
   test.fifo_controller.fcounter = 'h0;
   test.fifo_controller.last_subword = 'b0;
   test.fifo_controller.full = 1;
   test.fifo_controller.empty = 1;
   GSR = 1;
   #83 GSR = 0;

   $display("      time  C  R  R R   R  R L   R   W  W    W       W      F E  F  F  F");
   $display("            L  D  A A   A  A S   D   R  A    A       D      U M  C  I  C");
   $display("            O  E  L L   D  D B   A   E  L    D       A      L P  A  F  N");
   $display("            C  N  O M   D  M W   T   N  O    D       T      L T  L  O  T");
   $display("            K     W N   R  N D   A      W    R       A        Y  W  C   ");
   $display("");
   $monitor(" %9.2f  %b  %b  %b %b  %3x  %x %b  %3x  %b  %b  %3x %3x%3x%3x%3x %b %b  %b  %x %3x",
             $realtime, clockin, read_enable, test.fifo_controller.read_allow,
             test.fifo_controller.read_allow_minor,
             test.fifo_controller.read_addr, 
             test.fifo_controller.read_addr_minor, 
             test.fifo_controller.last_subword,
             read_data, write_enable, test.fifo_controller.write_allow, 
             test.fifo_controller.write_addr, write_data[35:27], write_data[26:18], 
             write_data[17:9], write_data[8:0], full, empty, 
             test.fifo_controller.fcnt_allow, fifocount,
             test.fifo_controller.fcounter
             );
end

initial
   clockin = 1;
   always #10 clockin = ~clockin;

task delay;
   begin
      @(posedge clockin) temp = 0;
      @(posedge clockin) temp = 0;
      @(posedge clockin) temp = 0;
      @(posedge clockin) temp = 0;
      @(posedge clockin) temp = 0;
   end
endtask

task writeburst;
   input [35:0] wdata;
   begin
      @(posedge clockin) begin 
         write_enable = #2 1;
         write_data = #2 wdata;
      end
   end
endtask

task writeburst32;
   begin
      writeburst(36'h000444433); writeburst(36'h088886644); writeburst(36'h110cc8855);
      writeburst(36'h19910aa66); writeburst(36'h22154cc77); writeburst(36'h2a998ee88);
      writeburst(36'h331dd1099); writeburst(36'h3ba2132aa); writeburst(36'h4426554bb);
      writeburst(36'h4caa976cc); writeburst(36'h552ed98dd); writeburst(36'h5db31baee);
      writeburst(36'h66375dcff); writeburst(36'h6ebb9fe00); writeburst(36'h773fc0011);
      writeburst(36'h7f8002222);
      writeburst(36'h88c8a2322); writeburst(36'h914ce4533); writeburst(36'h99d126744);
      writeburst(36'ha25568955); writeburst(36'haad9aab66); writeburst(36'hb35decd77);
      writeburst(36'hbbe22ef88); writeburst(36'hc46671199); writeburst(36'hcceab33aa);
      writeburst(36'hd56ef55bb); writeburst(36'hddf3377cc); writeburst(36'he677799dd);
      writeburst(36'heefbbbbee); writeburst(36'hf77ffddff); writeburst(36'hffc03ff00);
      writeburst(36'h804460111);
  end
endtask

task writeburst128;
   begin 
      writeburst32; writeburst32; writeburst32; writeburst32;
  end
endtask

task endwriteburst;
   begin
      @(posedge clockin) begin 
         write_enable = #2 0;
         write_data = #2 'h0;
      end
   end
endtask

task readburst;
   begin
      @(posedge clockin) #2 read_enable = 1;
   end
endtask

task readburst32;
   begin
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst;
   end
endtask
 
task readburst128;
   begin
      readburst32; readburst32; readburst32; readburst32;
   end
endtask
 
task endreadburst;
   begin
      @(posedge clockin) #2 read_enable = 0;
   end
endtask

initial begin

   delay;
   writeburst128; 
   writeburst128; 
   writeburst128; 
   writeburst128; 
   writeburst(36'h552ed98dd); 
   read_enable = 1;
   writeburst128; 
   endwriteburst; 
   read_enable = 0;
   delay; 
   readburst128; 
   writeburst128; 
   endreadburst;
   endwriteburst; 
   delay; 
   readburst128; 
   readburst128; 
   readburst128; 
   readburst128; 
   readburst128; 
   readburst128; 
   readburst128; 
   readburst128; 
   readburst128; 
   readburst128; 
   readburst128; 
   readburst128; 
   readburst128; 
   readburst128; 
   readburst128; 
   readburst128; 
   readburst128; 
   endreadburst;

end

endmodule

