
/**********************************************************************\
*                                                                      *
* Module      : tb_x261v_ccmw2.v              Last updated: 07/14/00   * 
*                                                                      *
* Description : Testbench for common-clock FIFO controller in a        * 
*               Virtex2 device, using source fifoctlr_ccmw2_v2,v,      * 
*               which implements a 511x36 Read, 2044x9 Write FIFO.     *
*                                                                      *
*  Designer    : Nick Camilleri                                        *
*                                                                      *
*  Company     : Xilinx, Inc.                                          *
*                                                                      *
*  Disclaimer  : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY   *
*                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY      *
*                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR    *
*                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.        *
*                THEY ARE ONLY INTENDED TO BE USED BY XILINX           *
*                CUSTOMERS, AND WITHIN XILINX DEVICES.                 *
*                                                                      *
*                Copyright (c) 2000 Xilinx, Inc.                       *
*                All rights reserved                                   *
*                                                                      *
\**********************************************************************/

`timescale 1ns / 10ps

module test;

reg GSR;
`define GSR_SIGNAL test.GSR

reg clockin;
reg read_enable, write_enable;
wire [35:0] read_data;
reg [8:0] write_data;
reg temp;

wire full, empty;
wire [3:0] fifocount;

fifoctlr_ccmw2_v2 fifo_controller (.clock_in(clockin), 
                       .read_enable_in(read_enable), .read_data_out(read_data), 
                       .write_data_in(write_data), .write_enable_in(write_enable),
                       .full_out(full), .empty_out(empty), 
                       .fifocount_out(fifocount), .fifo_gsr_in(GSR) );

initial begin
   read_enable = 0;
   write_enable = 0;
   write_data = 'h0;
   test.fifo_controller.read_addr = 'h0;
   test.fifo_controller.write_addr_minor = 'h0;
   test.fifo_controller.write_addr = 'h0;
   test.fifo_controller.fcounter = 'h0;
   test.fifo_controller.last_subword = 'b0;
   test.fifo_controller.full = 1;
   test.fifo_controller.empty = 1;
   GSR = 1;
   #83 GSR = 0;

   $display("      time  C  R  R   R        R       W  W W   W  W L   W   F E  F  F  F");
   $display("            L  D  A   A        D       R  A A   A  A S   D   U M  C  I  C");
   $display("            O  E  L   D        A       E  L L   D  D B   A   L P  A  F  N");
   $display("            C  N  O   D        T       N  O M   D  M W   T   L T  L  O  T");
   $display("            K     W   R        A          W N   R  N D   A     Y  W  C   ");
   $display("");
   $monitor(" %9.2f  %b  %b  %b  %3x %3x%3x%3x%3x  %b  %b %b  %3x %x %b  %3x  %b %b  %b  %x %3x",
             $realtime, clockin, read_enable, test.fifo_controller.read_allow,
             test.fifo_controller.read_addr, read_data[35:27], read_data[26:18], 
             read_data[17:9], read_data[8:0], write_enable, 
             test.fifo_controller.write_allow,
             test.fifo_controller.write_allow_minor,
             test.fifo_controller.write_addr, 
             test.fifo_controller.write_addr_minor, 
             test.fifo_controller.last_subword, write_data, full, empty, 
             test.fifo_controller.fcnt_allow, fifocount, 
             test.fifo_controller.fcounter
             );
end

initial
   clockin = 1;
   always #10 clockin = ~clockin;

task delay;
   begin
      @(posedge clockin) temp = 0;
      @(posedge clockin) temp = 0;
      @(posedge clockin) temp = 0;
      @(posedge clockin) temp = 0;
      @(posedge clockin) temp = 0;
   end
endtask

task writeburst;
   input [8:0] wdata;
   begin
      @(posedge clockin) begin 
         write_enable = #2 1;
         write_data = #2 wdata;
      end
   end
endtask

task writeburst32;
   begin 
      writeburst(9'h33); writeburst(9'h22); writeburst(9'h11); writeburst(9'h00);
      writeburst(9'h44); writeburst(9'h33); writeburst(9'h22); writeburst(9'h11);
      writeburst(9'h55); writeburst(9'h44); writeburst(9'h33); writeburst(9'h22);
      writeburst(9'h66); writeburst(9'h55); writeburst(9'h44); writeburst(9'h33);
      writeburst(9'h77); writeburst(9'h66); writeburst(9'h55); writeburst(9'h44);
      writeburst(9'h88); writeburst(9'h77); writeburst(9'h66); writeburst(9'h55);
      writeburst(9'h99); writeburst(9'h88); writeburst(9'h77); writeburst(9'h66);
      writeburst(9'haa); writeburst(9'h99); writeburst(9'h88); writeburst(9'h77);
      writeburst(9'hbb); writeburst(9'haa); writeburst(9'h99); writeburst(9'h88);
      writeburst(9'hcc); writeburst(9'hbb); writeburst(9'haa); writeburst(9'h99);
      writeburst(9'hdd); writeburst(9'hcc); writeburst(9'hbb); writeburst(9'haa);
      writeburst(9'hee); writeburst(9'hdd); writeburst(9'hcc); writeburst(9'hbb);
      writeburst(9'hff); writeburst(9'hee); writeburst(9'hdd); writeburst(9'hcc);
      writeburst(9'h00); writeburst(9'hff); writeburst(9'hee); writeburst(9'hdd);
      writeburst(9'h11); writeburst(9'h00); writeburst(9'hff); writeburst(9'hee);
      writeburst(9'h22); writeburst(9'h11); writeburst(9'h00); writeburst(9'hff);

      writeburst(9'h122); writeburst(9'h111); writeburst(9'h122); writeburst(9'h111);
      writeburst(9'h133); writeburst(9'h122); writeburst(9'h133); writeburst(9'h122);
      writeburst(9'h144); writeburst(9'h133); writeburst(9'h144); writeburst(9'h133);
      writeburst(9'h155); writeburst(9'h144); writeburst(9'h155); writeburst(9'h144);
      writeburst(9'h166); writeburst(9'h155); writeburst(9'h166); writeburst(9'h155);
      writeburst(9'h177); writeburst(9'h166); writeburst(9'h177); writeburst(9'h166);
      writeburst(9'h188); writeburst(9'h177); writeburst(9'h188); writeburst(9'h177);
      writeburst(9'h199); writeburst(9'h188); writeburst(9'h199); writeburst(9'h188);
      writeburst(9'h1aa); writeburst(9'h199); writeburst(9'h1aa); writeburst(9'h199);
      writeburst(9'h1bb); writeburst(9'h1aa); writeburst(9'h1bb); writeburst(9'h1aa);
      writeburst(9'h1cc); writeburst(9'h1bb); writeburst(9'h1cc); writeburst(9'h1bb);
      writeburst(9'h1dd); writeburst(9'h1cc); writeburst(9'h1dd); writeburst(9'h1cc);
      writeburst(9'h1ee); writeburst(9'h1dd); writeburst(9'h1ee); writeburst(9'h1dd);
      writeburst(9'h1ff); writeburst(9'h1ee); writeburst(9'h1ff); writeburst(9'h1ee);
      writeburst(9'h100); writeburst(9'h1ff); writeburst(9'h100); writeburst(9'h1ff);
      writeburst(9'h111); writeburst(9'h100); writeburst(9'h111); writeburst(9'h100);
  end
endtask

task writeburst128;
   begin 
      writeburst32; writeburst32; writeburst32; writeburst32;
  end
endtask

task endwriteburst;
   begin
      @(posedge clockin) begin 
         write_enable = #2 0;
         write_data = #2 'h0;
      end
   end
endtask

task readburst;
   begin
      @(posedge clockin) #2 read_enable = 1;
   end
endtask

task readburst32;
   begin
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst;
   end
endtask
 
task readburst128;
   begin
      readburst32; readburst32; readburst32; readburst32;
   end
endtask
 
task endreadburst;
   begin
      @(posedge clockin) #2 read_enable = 0;
   end
endtask

initial begin

   delay;
/* writeburst(0); */
   writeburst128; 
   writeburst128; 
   writeburst128; 
   writeburst128; 
   endwriteburst; 
   delay; 
   readburst; 
   endreadburst; 
   writeburst(9'hdd);
   writeburst(9'hcc);
   writeburst(9'hbb);
   writeburst(9'haa);
   delay; 
   readburst; 
   writeburst32; 
   writeburst32; 
   writeburst32; 
   endwriteburst; 
   endreadburst;
   delay; 
/* writeburst(0); */
   writeburst(9'h1dd);
   writeburst(9'h1cc);
   writeburst(9'h1bb);
   writeburst(9'h1aa);
   endwriteburst;
   readburst128;
   readburst128;
   endreadburst;
   
end

endmodule

