
/**********************************************************************\
*                                                                      *
* Module      : tb_x261v_icmw1b.v             Last updated: 07/14/00   * 
*                                                                      *
* Description : Testbench for independent-clock FIFO controller in a   *
*               Virtex2 device, using source fifoctlr_icmw1_v2,v.      * 
*               which implements a 2044x9 Read, 511x36 Write FIFO.     * 
*                                                                      *
*               Read clock = 15Mhz, Write clock = 50Mhz.               *  
*                                                                      *
*  Designer    : Nick Camilleri                                        *
*                                                                      *
*  Company     : Xilinx, Inc.                                          *
*                                                                      *
*  Disclaimer  : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY   *
*                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY      *
*                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR    *
*                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.        *
*                THEY ARE ONLY INTENDED TO BE USED BY XILINX           *
*                CUSTOMERS, AND WITHIN XILINX DEVICES.                 *
*                                                                      *
*                Copyright (c) 2000 Xilinx, Inc.                       *
*                All rights reserved                                   *
*                                                                      *
\**********************************************************************/

`timescale 1ns / 10ps

module test;

reg GSR;
`define GSR_SIGNAL test.GSR

reg read_clockin, write_clockin;
reg read_enable, write_enable;
wire [8:0] read_data;
reg [35:0] write_data;

wire full, empty;
wire [3:0] fifostatus;

fifoctlr_icmw1_v2 fifo_controller (.read_clock_in(read_clockin), 
                       .write_clock_in(write_clockin), 
                       .read_enable_in(read_enable), .read_data_out(read_data), 
                       .write_enable_in(write_enable), .write_data_in(write_data),
                       .full_out(full), .empty_out(empty), 
                       .fifostatus_out(fifostatus), .fifo_gsr_in(GSR) );

initial begin
   read_enable = 0;
   write_enable = 0;
   write_data = 'h0;
   test.fifo_controller.read_addr = 'h0;
   test.fifo_controller.read_addr_minor = 'h0;
   test.fifo_controller.read_nextgray = 'b100000000;
   test.fifo_controller.read_addrgray = 'b100000001;
   test.fifo_controller.read_lastgray = 'b100000011;
   test.fifo_controller.read_truegray = 'h0;
   test.fifo_controller.rag_writesync = 'h0;
   test.fifo_controller.write_addrr = 'h0;
   test.fifo_controller.write_addr = 'h0;
   test.fifo_controller.write_nextgray = 'b100000000;
   test.fifo_controller.write_addrgray = 'b100000001;
   test.fifo_controller.fifostatus = 'h0;
   test.fifo_controller.full = 'b1;
   test.fifo_controller.empty = 'b1;
   GSR = 1;
   #83 GSR = 0;

   $display("      time  R R R R   R  R   R   W W W   W       W        F E  F   R   R   R   W   W   EF  R   R   W");
   $display("            C D A A   A  A   D   C R A   A       D        U M  S   A   A   A   A   A       T   A   A");
   $display("            L E L L   D  D   A   L E L   D       A        L P  T   N   D   L   N   D       G   _   D");
   $display("            K N O M   D  M   T   K N O   D       T        L T  A   G   G   G   G   G           W   _");
   $display("                W N   R  N   A       W   R       A          Y  T                               S   R");
   $display("");
   $monitor(" %9.2f  %b %b %b %b  %3x %x  %3x  %b %b %b  %3x %3x%3x%3x%3x  %b %b %3x %3x %3x %3x %3x %3x  %b%b %3x %3x %3x",
             $realtime, read_clockin, read_enable, 
             test.fifo_controller.read_allow,
             test.fifo_controller.read_allow_minor,
             test.fifo_controller.read_addr,
             test.fifo_controller.read_addr_minor, read_data, 
             write_clockin, write_enable, test.fifo_controller.write_allow,
             test.fifo_controller.write_addr, write_data[35:27], 
             write_data[26:18], write_data[17:9], write_data[8:0],
             test.fifo_controller.full,
             test.fifo_controller.empty,
             test.fifo_controller.fifostatus,
             test.fifo_controller.read_nextgray,
             test.fifo_controller.read_addrgray,
             test.fifo_controller.read_lastgray,
             test.fifo_controller.write_nextgray,
             test.fifo_controller.write_addrgray,
             test.fifo_controller.emptyg,
             test.fifo_controller.fullg,
             test.fifo_controller.read_truegray,
             test.fifo_controller.ra_writesync,
             test.fifo_controller.write_addrr
             );
end

initial begin
   read_clockin = 1;
   write_clockin = 1;
end

always #33 read_clockin = ~read_clockin;    // write faster than read
always #10 write_clockin = ~write_clockin;

task delay;
   begin
      #100;
   end
endtask

task writeburst;
   input [35:0] wdata;
   begin
      @(posedge write_clockin) begin 
         write_enable = #2 1;
         write_data = #2 wdata;
      end
   end
endtask

task writeburst32;
   begin
      writeburst(36'h000444433); writeburst(36'h088886644); writeburst(36'h110cc8855);
      writeburst(36'h19910aa66); writeburst(36'h22154cc77); writeburst(36'h2a998ee88);
      writeburst(36'h331dd1099); writeburst(36'h3ba2132aa); writeburst(36'h4426554bb);
      writeburst(36'h4caa976cc); writeburst(36'h552ed98dd); writeburst(36'h5db31baee);
      writeburst(36'h66375dcff); writeburst(36'h6ebb9fe00); writeburst(36'h773fc0011);
      writeburst(36'h7f8002222);
      writeburst(36'h88c8a2322); writeburst(36'h914ce4533); writeburst(36'h99d126744);
      writeburst(36'ha25568955); writeburst(36'haad9aab66); writeburst(36'hb35decd77);
      writeburst(36'hbbe22ef88); writeburst(36'hc46671199); writeburst(36'hcceab33aa);
      writeburst(36'hd56ef55bb); writeburst(36'hddf3377cc); writeburst(36'he677799dd);
      writeburst(36'heefbbbbee); writeburst(36'hf77ffddff); writeburst(36'hffc03ff00);
      writeburst(36'h804460111);
  end
endtask

task writeburst128;
   begin 
      writeburst32; writeburst32; writeburst32; writeburst32;
  end
endtask

task endwriteburst;
   begin
      @(posedge write_clockin) begin 
         write_enable = #2 0;
         write_data = #2 'h0;
      end
   end
endtask

task readburst;
   begin
      @(posedge read_clockin) #2 read_enable = 1;
   end
endtask

task readburst32;
   begin
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst;
   end
endtask
 
task readburst128;
   begin
      readburst32; readburst32; readburst32; readburst32;
   end
endtask
 
task endreadburst;
   begin
      @(posedge read_clockin) #2 read_enable = 0;
   end
endtask

initial begin

// Stimulus when write faster than read

   delay;
   writeburst128;
   writeburst128;
   writeburst128;
   writeburst128;
   delay;
   read_enable = 1;
   writeburst128;
   endwriteburst;
   readburst128;
   readburst128;
   readburst128;
   writeburst128;
   endwriteburst;
   readburst128;
   readburst128;
   readburst128;
   readburst128;
   readburst128;
   readburst128;
   readburst128;
   readburst128;
   readburst128;
   readburst128;
   readburst128;
   readburst128;
   readburst128;
   readburst128;
   readburst128;
   readburst128;
   endreadburst;

end

endmodule

