
/**********************************************************************\
*                                                                      *
* Module      : tb_x261v_icmw2.v              Last updated: 07/14/00   * 
*                                                                      *
* Description : Testbench for independent-clock FIFO controller in a   *
*               Virtex2 device, using source fifoctlr_icmw2_v2,v.      * 
*               which implements a 511x36 Read, 2044x9 Write FIFO.     * 
*                                                                      *
*               Read clock = 50Mhz, Write clock = 15Mhz.               *  
*                                                                      *
*  Designer    : Nick Camilleri                                        *
*                                                                      *
*  Company     : Xilinx, Inc.                                          *
*                                                                      *
*  Disclaimer  : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY   *
*                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY      *
*                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR    *
*                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.        *
*                THEY ARE ONLY INTENDED TO BE USED BY XILINX           *
*                CUSTOMERS, AND WITHIN XILINX DEVICES.                 *
*                                                                      *
*                Copyright (c) 2000 Xilinx, Inc.                       *
*                All rights reserved                                   *
*                                                                      *
\**********************************************************************/

`timescale 1ns / 10ps

module test;

reg GSR;
`define GSR_SIGNAL test.GSR

reg read_clockin, write_clockin;
reg read_enable, write_enable;
wire [35:0] read_data;
reg [8:0] write_data;

wire full, empty;
wire [3:0] fifostatus;

fifoctlr_icmw2_v2 fifo_controller (.read_clock_in(read_clockin), 
                       .write_clock_in(write_clockin), 
                       .read_enable_in(read_enable), .read_data_out(read_data), 
                       .write_enable_in(write_enable), .write_data_in(write_data),
                       .full_out(full), .empty_out(empty), 
                       .fifostatus_out(fifostatus), .fifo_gsr_in(GSR) );

initial begin
   read_enable = 0;
   write_enable = 0;
   write_data = 'h0;
   test.fifo_controller.read_addr = 'h0;
   test.fifo_controller.read_nextgray = 'b100000000;
   test.fifo_controller.read_addrgray = 'b100000001;
   test.fifo_controller.read_lastgray = 'b100000011;
   test.fifo_controller.read_truegray = 'h0;
   test.fifo_controller.rag_writesync = 'h0;
   test.fifo_controller.write_addrr = 'h0;
   test.fifo_controller.write_addr = 'h0;
   test.fifo_controller.write_addr_minor = 'h0;
   test.fifo_controller.write_nextgray = 'b100000000;
   test.fifo_controller.write_addrgray = 'b100000001;
   test.fifo_controller.fifostatus = 'h0;
   test.fifo_controller.full = 'b1;
   test.fifo_controller.empty = 'b1;
   GSR = 1;
   #83 GSR = 0;

   $display("      time  R R R  R        R       W W W W  W  W  W   F E  F   R   R   R   W   W  EF   R   R   W");
   $display("            C D A  A        D       C R A A  A  A  D   U M  S   A   A   A   A   A       T   A   A");
   $display("            L E L  D        A       L E L L  D  D  A   L P  T   N   D   L   N   D       G   _   D");
   $display("            K N O  D        T       K N O M  D  M  T   L T  A   G   G   G   G   G           W   _");
   $display("                W  R        A           W N  R  N  A     Y  T                               S   R");
   $display("");
   $monitor(" %9.2f  %b %b %b %3x %3x%3x%3x%3x  %b %b %b %b %3x %x %3x  %b %b %3x %3x %3x %3x %3x %3x  %b%b %3x %3x %3x",
             $realtime, read_clockin, read_enable,
             test.fifo_controller.read_allow,
             test.fifo_controller.read_addr, 
             read_data[35:27], read_data[26:18], read_data[17:9], 
             read_data[8:0], write_clockin, write_enable, 
             test.fifo_controller.write_allow,
             test.fifo_controller.write_allow_minor,
             test.fifo_controller.write_addr, 
             test.fifo_controller.write_addr_minor, write_data,
             test.fifo_controller.full,
             test.fifo_controller.empty,
             test.fifo_controller.fifostatus,
             test.fifo_controller.read_nextgray,
             test.fifo_controller.read_addrgray,
             test.fifo_controller.read_lastgray,
             test.fifo_controller.write_nextgray,
             test.fifo_controller.write_addrgray,
             test.fifo_controller.emptyg,
             test.fifo_controller.fullg,
             test.fifo_controller.read_truegray,
             test.fifo_controller.ra_writesync,
             test.fifo_controller.write_addrr
             );
end

initial begin
   read_clockin = 1;
   write_clockin = 1;
end

always #10 read_clockin = ~read_clockin;    // read faster than write
always #33 write_clockin = ~write_clockin;

task delay;
   begin
      #100;
   end
endtask

task writeburst;
   input [8:0] wdata;
   begin
      @(posedge write_clockin) begin 
         write_enable = #2 1;
         write_data = #2 wdata;
      end
   end
endtask

task writeburst32;
   begin 
      writeburst(9'h33); writeburst(9'h22); writeburst(9'h11); writeburst(9'h00);
      writeburst(9'h44); writeburst(9'h33); writeburst(9'h22); writeburst(9'h11);
      writeburst(9'h55); writeburst(9'h44); writeburst(9'h33); writeburst(9'h22);
      writeburst(9'h66); writeburst(9'h55); writeburst(9'h44); writeburst(9'h33);
      writeburst(9'h77); writeburst(9'h66); writeburst(9'h55); writeburst(9'h44);
      writeburst(9'h88); writeburst(9'h77); writeburst(9'h66); writeburst(9'h55);
      writeburst(9'h99); writeburst(9'h88); writeburst(9'h77); writeburst(9'h66);
      writeburst(9'haa); writeburst(9'h99); writeburst(9'h88); writeburst(9'h77);
      writeburst(9'hbb); writeburst(9'haa); writeburst(9'h99); writeburst(9'h88);
      writeburst(9'hcc); writeburst(9'hbb); writeburst(9'haa); writeburst(9'h99);
      writeburst(9'hdd); writeburst(9'hcc); writeburst(9'hbb); writeburst(9'haa);
      writeburst(9'hee); writeburst(9'hdd); writeburst(9'hcc); writeburst(9'hbb);
      writeburst(9'hff); writeburst(9'hee); writeburst(9'hdd); writeburst(9'hcc);
      writeburst(9'h00); writeburst(9'hff); writeburst(9'hee); writeburst(9'hdd);
      writeburst(9'h11); writeburst(9'h00); writeburst(9'hff); writeburst(9'hee);
      writeburst(9'h22); writeburst(9'h11); writeburst(9'h00); writeburst(9'hff);
 
      writeburst(9'h122); writeburst(9'h111); writeburst(9'h122); writeburst(9'h111);
      writeburst(9'h133); writeburst(9'h122); writeburst(9'h133); writeburst(9'h122);
      writeburst(9'h144); writeburst(9'h133); writeburst(9'h144); writeburst(9'h133);
      writeburst(9'h155); writeburst(9'h144); writeburst(9'h155); writeburst(9'h144);
      writeburst(9'h166); writeburst(9'h155); writeburst(9'h166); writeburst(9'h155);
      writeburst(9'h177); writeburst(9'h166); writeburst(9'h177); writeburst(9'h166);
      writeburst(9'h188); writeburst(9'h177); writeburst(9'h188); writeburst(9'h177);
      writeburst(9'h199); writeburst(9'h188); writeburst(9'h199); writeburst(9'h188);
      writeburst(9'h1aa); writeburst(9'h199); writeburst(9'h1aa); writeburst(9'h199);
      writeburst(9'h1bb); writeburst(9'h1aa); writeburst(9'h1bb); writeburst(9'h1aa);
      writeburst(9'h1cc); writeburst(9'h1bb); writeburst(9'h1cc); writeburst(9'h1bb);
      writeburst(9'h1dd); writeburst(9'h1cc); writeburst(9'h1dd); writeburst(9'h1cc);
      writeburst(9'h1ee); writeburst(9'h1dd); writeburst(9'h1ee); writeburst(9'h1dd);
      writeburst(9'h1ff); writeburst(9'h1ee); writeburst(9'h1ff); writeburst(9'h1ee);
      writeburst(9'h100); writeburst(9'h1ff); writeburst(9'h100); writeburst(9'h1ff);
      writeburst(9'h111); writeburst(9'h100); writeburst(9'h111); writeburst(9'h100);
  end
endtask

task writeburst128;
   begin 
      writeburst32; writeburst32; writeburst32; writeburst32;
  end
endtask

task endwriteburst;
   begin
      @(posedge write_clockin) begin 
         write_enable = #2 0;
         write_data = #2 'h0;
      end
   end
endtask

task readburst;
   begin
      @(posedge read_clockin) #2 read_enable = 1;
   end
endtask

task readburst32;
   begin
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst; readburst; readburst; readburst; readburst;
      readburst; readburst;
   end
endtask

task readburst128;
   begin
      readburst32; readburst32; readburst32; readburst32;
   end
endtask

task endreadburst;
   begin
      @(posedge read_clockin) #2 read_enable = 0;
   end
endtask

initial begin

// Stimulus when read faster than write

   delay;
   writeburst128;
   writeburst128;
   writeburst128;
   writeburst128;
   endwriteburst;
   delay;
   read_enable = 1;
   writeburst32;
   endwriteburst;
   delay;
   readburst128;
   read_enable = 0;

end

endmodule

